@@ -4,11 +4,9 @@ import typing as t
 
 import logging
 
-from contextlib import asynccontextmanager
-from contextvars import ContextVar
 from urllib.parse import urlsplit
 
-from . import logger
+from . import logger, current_conn
 from .backends import BACKENDS, SHORTCUTS, ABCDatabaseBackend, ABCConnection, ABCTransaction
 
 
@@ -33,10 +31,9 @@ class Database:
         self.url = url
         self.logger = logger
         self.backend: ABCDatabaseBackend = backend_cls(parsed_url, logger=self.logger, **options)
-        self._conn_ctx: ContextVar = ContextVar('connection', default=None)
 
     async def connect(self) -> Database:
-        """Connect the database."""
+        """Open the database's pool."""
         if not self.is_connected:
             self.logger.info(f'Database connect: {self.url}')
             await self.backend.connect()
@@ -47,85 +44,107 @@ class Database:
     __aenter__ = connect
 
     async def disconnect(self, *args) -> None:
-        """Disconnect the database."""
-        conn = self._conn_ctx.get(None)
-        if conn is not None:
-            await conn.release()
+        """Close connections and the database's pool."""
+        self.logger.info(f'Database disconnect: {self.url}')
+
+        # Release connections
+        #  while self.current_conn:
+        #      await self.current_conn.release()
 
         if self.is_connected:
-            self.logger.info(f'Database disconnect: {self.url}')
             await self.backend.disconnect()
             self.is_connected = False
 
-        self._conn_ctx.set(None)
-
     __aexit__ = disconnect
 
-    def __create_connection__(self) -> ABCConnection:
-        """Create a connection."""
-        conn = self.backend.connection()
-        self._conn_ctx.set(conn)
-        return conn
+    @property
+    def current_conn(self) -> t.Optional[ABCConnection]:
+        return current_conn.get()
 
-    def __get_connection__(self, force_create: bool = True) -> t.Tuple[ABCConnection, bool]:
+    def connection(self, create: bool = True) -> ConnectionContext:
         """Get/create a connection from/to the current context."""
-        created = force_create
-        conn = self._conn_ctx.get()
-        if conn is None or force_create:
-            created = True
-            conn = self.__create_connection__()
-        return conn, created
+        return ConnectionContext(self.backend, use_existing=not create)
 
-    def connection(self, create: bool = True) -> ABCConnection:
-        """Get/create a connection from/to the current context."""
-        conn, _ = self.__get_connection__(create)
-        return conn
-
-    def transaction(self, create: bool = False) -> ABCTransaction:
+    def transaction(self, create: bool = False) -> TransactionContext:
         """Create a transaction."""
-        return self.connection(create).transaction()
+        return TransactionContext(self.backend, use_existing=not create)
 
     async def execute(self, query: t.Any, *params, **options) -> t.Any:
         """Execute a query."""
-        async with manage_connection(*self.__get_connection__(False)) as conn:
+        async with self.connection(False) as conn:
             return await conn.execute(query, *params)
 
     async def executemany(self, query: t.Any, *params, **options) -> t.Any:
         """Execute a query many times."""
-        async with manage_connection(*self.__get_connection__(False)) as conn:
+        async with self.connection(False) as conn:
             return await conn.executemany(query, *params, **options)
 
     async def fetchall(self, query: t.Any, *params, **options) -> t.List[t.Mapping]:
         """Fetch all rows."""
-        async with manage_connection(*self.__get_connection__(False)) as conn:
+        async with self.connection(False) as conn:
             return await conn.fetchall(query, *params, **options)
 
     async def fetchmany(self, size: int, query: t.Any, *params, **options) -> t.List[t.Mapping]:
         """Fetch rows."""
-        async with manage_connection(*self.__get_connection__(False)) as conn:
+        async with self.connection(False) as conn:
             return await conn.fetchmany(size, query, *params, **options)
 
     async def fetchone(self, query: t.Any, *params, **options) -> t.Optional[t.Mapping]:
         """Fetch a row."""
-        async with manage_connection(*self.__get_connection__(False)) as conn:
+        async with self.connection(False) as conn:
             return await conn.fetchone(query, *params)
 
     async def fetchval(self, query: t.Any, *params, column: t.Any = 0, **options) -> t.Any:
         """Fetch a value."""
-        async with manage_connection(*self.__get_connection__(False)) as conn:
+        async with self.connection(False) as conn:
             return await conn.fetchval(query, *params, column=column, **options)
 
     async def iterate(self, query: t.Any, *params, **options) -> t.AsyncIterator:
         """Iterate through results."""
-        async with manage_connection(*self.__get_connection__(False)) as conn:
+        async with self.connection(False) as conn:
             async for res in conn.iterate(query, *params, **options):
                 yield res
 
 
-@asynccontextmanager
-async def manage_connection(connection: ABCConnection, release: bool = True):
-    """Acquire/release connections."""
-    await connection.acquire()
-    yield connection
-    if release:
-        await connection.release()
+class ConnectionContext:
+
+    __slots__ = 'release', 'conn', 'token'
+
+    def __init__(self, backend: ABCDatabaseBackend, *, use_existing: bool = False):
+        conn = current_conn.get()
+        if conn is None or not use_existing:
+            conn = backend.connection()
+            self.release = True
+        else:
+            self.release = False
+
+        self.conn = conn
+
+    async def __aenter__(self):
+        conn = self.conn
+        await conn.acquire()
+        self.token = current_conn.set(conn)
+        return conn
+
+    async def __aexit__(self, *args):
+        if self.release:
+            current_conn.reset(self.token)
+            await self.conn.release()
+
+
+class TransactionContext(ConnectionContext):
+
+    __slots__ = ConnectionContext.__slots__ + ('trans',)
+
+    def __init__(self, backend: ABCDatabaseBackend, *, use_existing: bool = True):
+        super(TransactionContext, self).__init__(backend, use_existing=use_existing)
+        self.trans = self.conn.transaction()
+
+    async def __aenter__(self):
+        await super(TransactionContext, self).__aenter__()
+        await self.trans.__aenter__()
+        return self.trans
+
+    async def __aexit__(self, *args):
+        await self.trans.__aexit__(*args)
+        await super(TransactionContext, self).__aexit__(*args)
